use std::{os::unix::ffi::OsStrExt, path::Path, sync::OnceLock};

static ROOT_DB: OnceLock<sled::Db> = OnceLock::new();

#[derive(Clone)]
pub struct FileCache {
    tree: sled::Tree,
}

async fn get_db_mutex() -> &'static sled::Db {
    ROOT_DB.get_or_init(|| {
        sled::open(
            dirs::config_dir()
                .unwrap()
                .join(crate::CONFIG_DIR_NAME)
                .join("synced.db"),
        )
        .expect("Couldn't open DB to store synced files")
    })
}

impl FileCache {
    pub async fn open_for_device(id: impl AsRef<str>) -> Self {
        let tree = get_db_mutex().await.open_tree(id.as_ref()).unwrap();
        Self { tree }
    }

    pub async fn store(&self, path: impl AsRef<Path>) -> Result<(), sled::Error> {
        let path = path.as_ref().canonicalize().unwrap();
        let path_bytes = path.as_os_str().as_bytes();
        if !self.tree.contains_key(path_bytes)? {
            self.tree.insert(path.as_os_str().as_bytes(), b"")?;
        }
        Ok(())
    }

    pub fn has_path(&self, path: impl AsRef<Path>) -> Result<bool, sled::Error> {
        let path = path.as_ref().canonicalize().unwrap();
        let path_bytes = path.as_os_str().as_bytes();
        self.tree.contains_key(path_bytes)
    }
}
